@@ -9,7 +9,6 @@ OSGi::Osgish::Agent - Access to the OSGi agent bundle
 =head1 DESCRIPTION
 
 =cut
-
 package OSGi::Osgish::Agent;
 
 use strict;
@@ -18,7 +17,7 @@ use JMX::Jmx4Perl::Request;
 use OSGi::Osgish::Agent::Upload;
 use Data::Dumper;
 
-
+# Osgi EEG MBeans
 my $MBEANS_MAP = 
     { 
      "framework" => { key => "type", version => "1.5", domain => "osgi.core" },
@@ -31,6 +30,7 @@ my $MBEANS_MAP =
      "useradmin" => { key => "service", version => "1.1", domain => "osgi.compendium" }
     };
 
+# Our own MBean for certain state checks
 my $OSGISH_SERVICE_NAME = "osgish:type=Service";
 
 sub new { 
@@ -113,7 +113,7 @@ sub bundle_symbolic_names {
 sub bundle_ids {
     my $self = shift;
     $self->_update_bundles(@_);
-    return sort keys %{$self->{bundle}->{ids}};
+    return [ sort keys %{$self->{bundle}->{ids}} ];
 }
 
 sub bundle_name {
@@ -170,7 +170,7 @@ sub uninstall_bundle {
 sub refresh_bundle {
     my $self = shift;
     $self->_update_bundles();
-    $self->_bulk_bundle_cmd("refreshPackages(long)","refreshPackages([J)",@_);
+    $self->_bulk_bundle_cmd("refreshBundle","refreshBundles",@_);
 }
 
 sub _bulk_bundle_cmd {
@@ -183,9 +183,10 @@ sub _bulk_bundle_cmd {
     }
     die "No id given\n" unless @ids;
     if (@ids > 1) {
-        $self->execute($self->_mbean_name("framework"),$what_multi,\@ids);
+        return $self->execute($self->_mbean_name("framework"),$what_multi,\@ids);
     } else {
         $self->execute($self->_mbean_name("framework"),$what_single,$ids[0]);
+        return $ids[0];
     }
 }
 
@@ -194,7 +195,9 @@ sub install_bundle {
     my @locations = @_;
     my $mbean = $self->_mbean_name("framework");
     if (@locations > 1) {
-        $self->execute($mbean,"installBundles([Ljava.lang.String;)",\@locations);
+        # Must be posted due to the ";" sign, which, even when escaped,
+        # confuses e.g. Glassfish when part of an GET Url
+        $self->execute_post($mbean,"installBundles([Ljava.lang.String;)",\@locations);
     } else {
         $self->execute($mbean,"installBundle(java.lang.String)",$locations[0]);
     }
@@ -210,6 +213,7 @@ sub update_bundle {
     } else {
         $self->execute($self->_mbean_name("framework"),"updateBundle(long)",$id);
     }
+    return $id;
 }
 
 # Return values
@@ -252,14 +256,29 @@ sub execute {
     my $operation = shift || die "No operation given for MBean $mbean";
     my @args = @_;
 
+    return $self->_do_execute(new JMX::Jmx4Perl::Request(EXEC,$mbean,$operation,@args));
+}
+
+sub execute_post {
+    my $self = shift;
+    my $mbean = shift || die "No MBean name given";
+    my $operation = shift || die "No operation given for MBean $mbean";
+    my @args = @_;
+
+    return $self->_do_execute(new JMX::Jmx4Perl::Request(EXEC,$mbean,$operation,@args,{method => "POST"}));
+}
+
+sub _do_execute {
+    my $self = shift;
+    my $request = shift;
+
     my $j4p = $self->{j4p};
 
-    my $request = new JMX::Jmx4Perl::Request(EXEC,$mbean,$operation,@args);
     my $response = $j4p->request($request);
     if ($response->is_error) {
         #print Dumper($response);
         if ($response->status == 404) {
-            die "No agent running [Not found: $mbean,$operation].\n"
+            die "No agent running [Not found: ",$request->{mbean},",",$request->{operation},"].\n"
         } else {
             $self->{last_error} = $response->{error} . 
               ($response->stacktrace ? "\nStacktrace:\n" . $response->stacktrace : "");
@@ -374,9 +393,10 @@ sub _fetch_packages {
     my $package = $self->_fetch_list("packageState","listPackages");
     $package->{import_export} = $self->_extract_import_export($package->{list});
     $self->{package} = $package;
+    #print Dumper($package);
 }
 
-sub exporting_bundle {
+sub exporting_bundles {
     return shift->_exporting_importing_bundle("exporting",@_);
 }
 
@@ -404,25 +424,25 @@ sub _extract_import_export {
         # We are using the chached bundle names here. Should be ok.
         $ret->{$v->{Name}}->{$v->{Version}} = 
             {              
-             importing => 
-             [ 
-              map { 
-                    { 
-                      id => $_, 
-                      name => $self->{bundle}->{ids}->{$_}
-                    }
-                } 
-              @{$v->{ImportingBundles}}
-             ],
-             exporting => { 
-                           id => $v->{ExportingBundle},
-                           name => $self->{bundle}->{ids}->{$v->{ExportingBundle}}
-                          }
+             importing => $self->_extract_unique_bundles($v->{ImportingBundles}),
+             exporting => $self->_extract_unique_bundles($v->{ExportingBundles})
             };
     }
     return $ret;
 }
 
+sub _extract_unique_bundles {
+    my $self = shift;
+    my $bundles = shift;
+    my $ret = {};
+    map { 
+        $ret->{$_} = { 
+                      id => $_,
+                      name => $self->{bundle}->{ids}->{$_}
+                     }
+    } @$bundles;
+    return [ sort { $a->{id} <=> $b->{id} } values %$ret ];
+}
 
 sub _fetch_list {
     my $self = shift;
@@ -461,4 +481,40 @@ sub _extract_object_classes {
     return ($cl,$ids);
 }
 
+=head1 LICENSE
+
+This file is part of osgish.
+
+Osgish is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 2 of the License, or
+(at your option) any later version.
+
+osgish is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with osgish.  If not, see <http://www.gnu.org/licenses/>.
+
+A commercial license is available as well. Please contact roland@cpan.org for
+further details.
+
+=head1 PROFESSIONAL SERVICES
+
+Just in case you need professional support for this module (or JMX or OSGi in
+general), you might want to have a look at www.consol.com Contact
+roland.huss@consol.de for further information (or use the contact form at
+http://www.consol.com/contact/)
+
+=head1 AUTHOR
+
+roland@cpan.org
+
+=cut
+
+
+
 1;
+
